/*-----------------------------------------------------------------------
  Name      : timer.c
  Author    : Jack Botner
  Platform  : Windows 32-bit
  Compiler  : Microsoft Visual C++ 2010
 Description: Calculator for AVR microcontroller timer parameters

  Change Log: 20220207 written
-----------------------------------------------------------------------*/
#undef UNICODE
#include <windows.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "resource.h"

#define INTERVAL_MICROSEC     1
#define INTERVAL_MILLISEC     2
#define INTERVAL_SECONDS      3

#define TIMER_8BIT            1
#define TIMER_16BIT           2

#define PRESCALER_SET1        1
#define PRESCALER_SET2        2
#define PRESCALER_SET3        3

// Pprogram parameters

typedef struct
{
  HINSTANCE     hInstance;              // program instance handle

  // Inputs
  unsigned long ulClockFreqHz;          // clock frequency Hz
  unsigned long ulInterval;             // desired interval
  short         sIntervalUnits;         // interval units
  short         sTimerSize;             // 8 or 16 bit timer
  short         sPrescalerSet;          // prescaler set
}
PARMS, *PPARMS;

// Global variables

// Internal functions

static BOOL accessDialog( HWND hwnd );
static BOOL computeValues( HWND hwnd );
static BOOL CALLBACK dialogMessages( HWND hwnd, UINT uMsg, WPARAM wParam,
                                     LPARAM lParam );

/*-------------------------------------------------------------------------
  WinMain(...)
-------------------------------------------------------------------------*/
int WINAPI WinMain( HINSTANCE hInst, HINSTANCE hPrev,
                    LPSTR lpArgs, int nWinMode )
{
  int                   iDlgRc;
  PARMS                 stParms;

  memset( &stParms, '\0', sizeof( PARMS ) );
  stParms.hInstance      = hInst;
  stParms.ulClockFreqHz  = 8000000L;
  stParms.ulInterval     = 1L;
  stParms.sIntervalUnits = INTERVAL_MILLISEC;
  stParms.sTimerSize     = TIMER_8BIT;  
  stParms.sPrescalerSet  = PRESCALER_SET2;

  iDlgRc = DialogBoxParam( hInst, MAKEINTRESOURCE( IDD_TIMER ), HWND_DESKTOP,
                           (DLGPROC) dialogMessages, (LPARAM) &stParms );

  return 0;
}

/*-------------------------------------------------------------------------
  dialogMessages(...)
-------------------------------------------------------------------------*/
static BOOL CALLBACK dialogMessages( HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam )
{
  PPARMS        pParms;
  BOOL          bRc, bRcode=FALSE;
  LONG          lRc;
  char          work[32];

  switch ( uMsg )
  {
    case WM_INITDIALOG:
      pParms = (PPARMS) lParam;        // get our parameters
      lRc = SetWindowLong( hwnd, DWL_USER, lParam );  // store it in the dialog

	  lRc = SendDlgItemMessage( hwnd, IDC_CLOCK, EM_LIMITTEXT, 8, 0 );
	  sprintf( work, "%lu", pParms->ulClockFreqHz );
      bRc = SetDlgItemText( hwnd, IDC_CLOCK, work );

	  lRc = SendDlgItemMessage( hwnd, IDC_INTERVAL, EM_LIMITTEXT, 8, 0 );
	  sprintf( work, "%lu", pParms->ulInterval );
      bRc = SetDlgItemText( hwnd, IDC_INTERVAL, work );

	  lRc = SendDlgItemMessage( hwnd, IDC_USEC, BM_SETCHECK, BST_UNCHECKED,0  );
      lRc = SendDlgItemMessage( hwnd, IDC_MILLI, BM_SETCHECK, BST_UNCHECKED,0  );
      lRc = SendDlgItemMessage( hwnd, IDC_SEC, BM_SETCHECK, BST_UNCHECKED, 0 );

      if ( pParms->sIntervalUnits == INTERVAL_MICROSEC )
		lRc = SendDlgItemMessage( hwnd, IDC_USEC, BM_SETCHECK, BST_CHECKED,0  );
	  else
        if ( pParms->sIntervalUnits == INTERVAL_MILLISEC )
          lRc = SendDlgItemMessage( hwnd, IDC_MILLI, BM_SETCHECK, BST_CHECKED,0  );
		else
          lRc = SendDlgItemMessage( hwnd, IDC_SEC, BM_SETCHECK, BST_CHECKED, 0 );

	  lRc = SendDlgItemMessage( hwnd, IDC_8BIT, BM_SETCHECK, BST_UNCHECKED,0  );
      lRc = SendDlgItemMessage( hwnd, IDC_16BIT, BM_SETCHECK, BST_UNCHECKED,0  );

      if ( pParms->sTimerSize == TIMER_8BIT )
		lRc = SendDlgItemMessage( hwnd, IDC_8BIT, BM_SETCHECK, BST_CHECKED,0  );
	  else
		lRc = SendDlgItemMessage( hwnd, IDC_16BIT, BM_SETCHECK, BST_CHECKED,0  );

      lRc = SendDlgItemMessage( hwnd, IDC_SET1, BM_SETCHECK, BST_UNCHECKED,0  );
	  lRc = SendDlgItemMessage( hwnd, IDC_SET2, BM_SETCHECK, BST_UNCHECKED,0  );
	  lRc = SendDlgItemMessage( hwnd, IDC_SET3, BM_SETCHECK, BST_UNCHECKED,0  );

	  if ( pParms->sPrescalerSet == PRESCALER_SET1 )
		lRc = SendDlgItemMessage( hwnd, IDC_SET1, BM_SETCHECK, BST_CHECKED,0  );
	  else
		if ( pParms->sPrescalerSet == PRESCALER_SET2 )
		  lRc = SendDlgItemMessage( hwnd, IDC_SET2, BM_SETCHECK, BST_CHECKED,0  );
		else
		  lRc = SendDlgItemMessage( hwnd, IDC_SET3, BM_SETCHECK, BST_CHECKED,0  );
	  break;

    case WM_COMMAND:
      switch ( LOWORD( wParam ) )
      {
        case IDOK:
          bRc = accessDialog( hwnd );
		  if ( bRc )
            bRc = computeValues( hwnd );
          break;

        case IDCANCEL:
          bRc = EndDialog( hwnd, 0 );
          break;
      }
      break;

  }  // end switch

  return bRcode;
}

/*-------------------------------------------------------------------------
  accessDialog(...)

  Gets users parameters from the dialog box.

  Returns: TRUE = OK, FALSE = error
-------------------------------------------------------------------------*/
static BOOL accessDialog( HWND hwnd )
{
  PPARMS        pParms;
  unsigned      uiRc;
  int           iRc;
  unsigned long ulNr;
  short         sNr;
  char          work[32];

  pParms = (PPARMS) GetWindowLong(  hwnd, DWL_USER );

  // Get the updated dialog paranmeters

  uiRc = GetDlgItemText( hwnd, IDC_CLOCK, work, sizeof( work ) );
  if ( uiRc )   // something returned
  {
    iRc = sscanf( work, "%lu", &ulNr );
    if ( iRc != 1 )
    {
      MessageBox( hwnd, "Invalid clock frequency value entered", "TIMER",
                    MB_OK | MB_ICONSTOP );
      SetFocus( GetDlgItem( hwnd, IDC_CLOCK ) );
      return FALSE;
    }
  }
  else    // nothing returned
  {
	MessageBox( hwnd, "Clock Frequency value missing", "TIMER",
                MB_OK | MB_ICONSTOP );
    SetFocus( GetDlgItem( hwnd, IDC_CLOCK ) );
    return FALSE;
  }

  if ( !ulNr || ulNr > 64000000L )
  {
    MessageBox( hwnd, "Invalid clock frequency value entered", "TIMER",
                MB_OK | MB_ICONSTOP );
    SetFocus( GetDlgItem( hwnd, IDC_CLOCK ) );
    return FALSE;
  }

  pParms->ulClockFreqHz = ulNr;

  uiRc = GetDlgItemText( hwnd, IDC_INTERVAL, work, sizeof( work ) );
  if ( uiRc )   // something returned
  {
    iRc = sscanf( work, "%lu", &ulNr );
    if ( iRc != 1 )
    {
      MessageBox( hwnd, "Invalid interval value entered", "TIMER",
                    MB_OK | MB_ICONSTOP );
      SetFocus( GetDlgItem( hwnd, IDC_INTERVAL ) );
      return FALSE;
    }
  }
  else    // nothing returned
  {
	MessageBox( hwnd, "Interval value missing", "TIMER",
                MB_OK | MB_ICONSTOP );
    SetFocus( GetDlgItem( hwnd, IDC_INTERVAL ) );
    return FALSE;
  }

  if ( !ulNr || ulNr > 64000000L )
  {
    MessageBox( hwnd, "Invalid interval value entered", "TIMER",
                MB_OK | MB_ICONSTOP );
    SetFocus( GetDlgItem( hwnd, IDC_INTERVAL ) );
    return FALSE;
  }

  pParms->ulInterval = ulNr;

  iRc = SendDlgItemMessage( hwnd, IDC_USEC, BM_GETCHECK, 0, 0 );
  if ( iRc == BST_CHECKED )
    sNr = INTERVAL_MICROSEC;
  else
  {
    iRc = SendDlgItemMessage( hwnd, IDC_MILLI, BM_GETCHECK, 0, 0 );
    if ( iRc == BST_CHECKED )
      sNr = INTERVAL_MILLISEC;
    else
      sNr = INTERVAL_SECONDS;
  }

  pParms->sIntervalUnits = sNr;

  iRc = SendDlgItemMessage( hwnd, IDC_8BIT, BM_GETCHECK, 0, 0 );
  if ( iRc == BST_CHECKED )
    sNr = TIMER_8BIT;
  else
    sNr = TIMER_16BIT;

  pParms->sTimerSize = sNr;

  iRc = SendDlgItemMessage( hwnd, IDC_SET1, BM_GETCHECK, 0, 0 );
  if ( iRc == BST_CHECKED )
    sNr = PRESCALER_SET1;
  else
  {
    iRc = SendDlgItemMessage( hwnd, IDC_SET2, BM_GETCHECK, 0, 0 );
    if ( iRc == BST_CHECKED )
      sNr = PRESCALER_SET2;
    else
      sNr = PRESCALER_SET3;
  }

  pParms->sPrescalerSet = sNr;

  return TRUE;
}

/*-------------------------------------------------------------------------
  computeValues(...)

  Once the inputs are set, computes and displays the outputs.


  Returns: TRUE = OK, FALSE = error
-------------------------------------------------------------------------*/
static BOOL computeValues( HWND hwnd )
{
  PPARMS        pParms;

  static short sSet1Tab[5] = { 1, 8, 64, 256, 1024 };
  static short nSet1 = 5;
  static short sSet2Tab[7] = { 1, 8, 32, 64, 128, 256, 1024 };
  static short nSet2 = 7;
  static short sSet3Tab[15] = { 1, 2, 4, 8, 16, 32, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 16384 };
  static short nSet3 = 15;

  short     sSetTab[16];
  short     nSet;
  short     ii;
  unsigned  nn;
  double    dMaxTimerCounts;
  double    dClockPeriodNs;
  double    dTimerFullCycleNs, dTimerFullCycleNsTmp;
  double    dDesiredIntervalNs;
  double    dTimeofaTickNs;
  BOOL      bRc;
  char      work[32];

  pParms = (PPARMS) GetWindowLong(  hwnd, DWL_USER );

  bRc = SetDlgItemText( hwnd, IDC_MSG, "" );
  bRc = SetDlgItemText( hwnd, IDC_COUNTS, "" );
  bRc = SetDlgItemText( hwnd, IDC_PRESEL, "" );

  // Set up the prescaler table
  if ( pParms->sPrescalerSet == PRESCALER_SET1 )
  {
	for ( ii=0; ii<nSet1; ++ii )
      sSetTab[ii] = sSet1Tab[ii];
	nSet = nSet1;
  }
  else
  {
    if ( pParms->sPrescalerSet == PRESCALER_SET2 )
    {
	  for ( ii=0; ii<nSet2; ++ii )
        sSetTab[ii] = sSet2Tab[ii];
	  nSet = nSet2;
    }
    else
    {
	  for ( ii=0; ii<nSet3; ++ii )
        sSetTab[ii] = sSet3Tab[ii];
	  nSet = nSet3;
    }
  }

  dMaxTimerCounts = ( pParms->sTimerSize == TIMER_8BIT ) ? 256.0 : 65536.0;

  dClockPeriodNs = 1000000000.0 / (double) pParms->ulClockFreqHz;

  dTimerFullCycleNs = dMaxTimerCounts * dClockPeriodNs;

  if ( pParms->sIntervalUnits == INTERVAL_MILLISEC )
    dDesiredIntervalNs = (double) pParms->ulInterval * 1000000.0;
  else
    if ( pParms->sIntervalUnits == INTERVAL_SECONDS )
      dDesiredIntervalNs = (double) pParms->ulInterval * 1000000000.0;
	else
	  dDesiredIntervalNs = (double) pParms->ulInterval * 1000.0;

  for ( ii=0; ii<nSet; ++ii )
  {
    dTimerFullCycleNsTmp = dTimerFullCycleNs * (double) sSetTab[ii];
    if ( dDesiredIntervalNs <= dTimerFullCycleNsTmp )
      break;
  }

  if ( ii == nSet )
  {
    bRc = SetDlgItemText( hwnd, IDC_MSG, "No solution" );
    return FALSE;
  }

  sprintf( work, "%u", (unsigned) sSetTab[ii] );
  bRc = SetDlgItemText( hwnd, IDC_PRESEL, work );

  dTimeofaTickNs = (double) sSetTab[ii] * dClockPeriodNs;
  nn =  (unsigned long) ( dDesiredIntervalNs / dTimeofaTickNs );

  sprintf( work, "%u", (unsigned) nn );
  bRc = SetDlgItemText( hwnd, IDC_COUNTS, work );

  return TRUE;
}